@@ -24,7 +24,7 @@ from .sampling_method import SamplingMethod
 from casadi import sumsqr, horzcat, vertcat, linspace, substitute, MX, evalf,\
                    vcat, collocation_points, collocation_interpolators, hcat,\
                    repmat, DM, sum2, mtimes, vvcat
-from .casadi_helpers import get_ranges_dict
+from .casadi_helpers import get_ranges_dict, HashOrderedDict
 from itertools import repeat
 try:
     from casadi import collocation_coeff
@@ -127,17 +127,26 @@ class DirectCollocation(SamplingMethod):
                 tr.append([self.integrator_grid[k][i]+dt*self.tau[j] for j in range(self.degree)])        
             self.tr.append(tr)
 
+        dts = []
+        # Fill in Z variables up-front, since they might be needed in constraints with ocp.next
         for k in range(self.N):
             dt = (self.control_grid[k + 1] - self.control_grid[k])/self.M
+            dts.append(dt)
             S = 1/repmat(hcat([dt**i for i in range(self.degree + 1)]), self.degree + 1, 1)
             S_z = 1/repmat(hcat([dt**i for i in range(self.degree)]), self.degree, 1)
             self.Z.append(mtimes(self.Zc[k][0],poly_z[:,0]))
-            p = self.get_p_sys(stage,k)
             for i in range(self.M):
                 self.xk.append(self.Xc[k][i][:,0])
+                self.zk.append(mtimes(self.Zc[k][i],poly_z[:,0]))
                 self.poly_coeff.append(mtimes(self.Xc[k][i],poly*S))
                 self.poly_coeff_z.append(mtimes(self.Zc[k][i],poly_z*S_z))
-                self.zk.append(mtimes(self.Zc[k][i],poly_z[:,0]))
+
+        self.Z.append(mtimes(self.Zc[-1][-1],sum2(poly_z)))
+
+        for k in range(self.N):
+            dt = dts[k]
+            p = self.get_p_sys(stage,k)
+            for i in range(self.M):
                 for j in range(self.degree):
                     Pidot_j = mtimes(self.Xc[k][i],self.C[:,j])/ dt
                     res = f(x=self.Xc[k][i][:, j+1], u=self.U[k], z=self.Zc[k][i][:,j], p=p, t=self.tr[k][i][j])
@@ -153,34 +162,53 @@ class DirectCollocation(SamplingMethod):
                 x_next = self.X[k + 1] if i==self.M-1 else self.Xc[k][i+1][:,0]
                 opti.subject_to(mtimes(self.Xc[k][i],self.D) == x_next)
 
-                for c, meta, _ in stage._constraints["integrator"]:
+                for c, meta, args in stage._constraints["integrator"]:
+                    if k==0 and i==0 and not args["include_first"]: continue
                     opti.subject_to(self.eval_at_integrator(stage, c, k, i), meta=meta)
                         
                 for c, meta, _ in stage._constraints["inf"]:
                     self.add_inf_constraints(stage, opti, c, k, i, meta)
 
-            for c, meta, _ in stage._constraints["control"]:  # for each constraint expression
+            for c, meta, args in stage._constraints["control"]:  # for each constraint expression
+                if k==0 and not args["include_first"]: continue
                 # Add it to the optimizer, but first make x,u concrete.
-                opti.subject_to(self.eval_at_control(stage, c, k), meta=meta)
+                try:
+                    opti.subject_to(self.eval_at_control(stage, c, k), meta=meta)
+                except IndexError:
+                    pass # Can be caused by ocp.offset -> drop constraint
 
-        self.Z.append(mtimes(self.Zc[-1][-1],sum2(poly_z)))
+        for c, meta, args in stage._constraints["control"]:  # for each constraint expression
+            if not args["include_last"]: continue
+            # Add it to the optimizer, but first make x,u concrete.
+            try:
+                opti.subject_to(self.eval_at_control(stage, c, -1), meta=meta)
+            except IndexError:
+                pass # Can be caused by ocp.offset -> drop constraint
 
-        for c, meta, _ in stage._constraints["control"]+stage._constraints["integrator"]:  # for each constraint expression
+        for c, meta, args in stage._constraints["integrator"]:  # for each constraint expression
+            if not args["include_last"]: continue
             # Add it to the optimizer, but first make x,u concrete.
             opti.subject_to(self.eval_at_control(stage, c, -1), meta=meta)
 
-    def set_initial(self, stage, opti, initial):
+    def set_initial(self, stage, master, initial):
+        opti = master.opti if hasattr(master, 'opti') else master
         opti.cache_advanced()
-        initial = dict(initial)
+        initial = HashOrderedDict(initial)
         algs = get_ranges_dict(stage.algebraics)
+        initial_alg = {}
         for a, v in list(initial.items()):
             if a in algs:
-                for k in range(self.N):
-                    for e in self.Zc[k]:
-                        e_shape = e[algs[a],:].shape
-                        opti.set_initial(e[algs[a],:], repmat(v,1,e_shape[1]))
+                initial_alg[a] = v
                 del initial[a]
         SamplingMethod.set_initial(self,stage, opti, initial)
+        for a, v in list(initial_alg.items()):
+            opti_initial = opti.initial()
+            for k in range(self.N):
+                value = DM(opti.debug.value(self.eval_at_control(stage, v, k), opti_initial))
+                for i, e in enumerate(self.Zc[k]):
+                    e_shape = e[algs[a],:].shape
+                    value = DM(opti.debug.value(hcat([self.eval_at_integrator_root(stage, v, k, i, j) for j in range(e_shape[1])]), opti_initial))                    
+                    opti.set_initial(e[algs[a],:], value)
         for k in range(self.N):
             x0 = DM(opti.debug.value(self.X[k], opti.initial()))
             for e in self.Xc[k]:
