@@ -24,6 +24,7 @@ from casadi import Opti, jacobian, dot, hessian, symvar, evalf, veccat, DM, vert
 import numpy as np
 from .casadi_helpers import get_meta, merge_meta, single_stacktrace, MX
 from .solution import OcpSolution
+from .freetime import FreeTime
 
 class DirectMethod:
     """
@@ -83,20 +84,22 @@ class DirectMethod:
             V.append(opti.variable(v.shape[0], v.shape[1]))
         self.V = veccat(*V)
 
-    def main_transcribe(self, stage, pass_nr=1, **kwargs):
-        if pass_nr==1:
+    def main_transcribe(self, stage, phase=1, **kwargs):
+        if phase==1:
             self.opti = OptiWrapper(stage)
             if self._callback:
                 self.opti.callback(self._callback)
             if self.solver is not None:
+                if self._solver is None:
+                    raise Exception("You forgot to declare a solver. Use e.g. ocp.solver('ipopt').")
                 self.opti.solver(self._solver, self._solver_options)
-        if pass_nr==2:
-            self.opti.transcribe_placeholders(kwargs["placeholders"])
+        if phase==2:
+            self.opti.transcribe_placeholders(phase, kwargs["placeholders"])
 
-    def transcribe(self, stage, pass_nr=1, **kwargs):
-        if pass_nr==2:
-            self.opti.transcribe_placeholders(kwargs["placeholders"])
-        if pass_nr>1: return
+    def transcribe(self, stage, phase=1, **kwargs):
+        if stage.nx>0 or stage.nu>0:
+            raise Exception("You forgot to declare a method. Use e.g. ocp.method(MultipleShooting(N=3)).")
+        if phase>1: return
         self.add_variables(stage, self.opti)
 
         for c, m, _ in stage._constraints["point"]:
@@ -113,10 +116,12 @@ class DirectMethod:
             value = DM(opti.debug.value(self.eval_top(stage, expr), opti_initial)) # HOT line
             opti.set_initial(target, value, cache_advanced=True)
 
-    def transcribe_placeholders(self, stage, placeholders):
+    def transcribe_placeholders(self, phase, stage, placeholders):
         pass
 
     def non_converged_solution(self, stage):
+        if not hasattr(self, 'opti'):
+            raise Exception("You forgot to solve first. To avoid your script halting, use a try-catch block.")
         return OcpSolution(self.opti.non_converged_solution, stage)
 
     def solve(self, stage):
@@ -136,8 +141,8 @@ class DirectMethod:
         self._solver = solver
         self._solver_options = solver_options
 
-    def show_infeasibilities(self, *args, **kwargs):
-        self.opti.debug.show_infeasibilities(*args, **kwargs)
+    def show_infeasibilities(self, *args):
+        self.opti.debug.show_infeasibilities(*args)
 
     @property
     def gist(self):
@@ -155,6 +160,38 @@ class DirectMethod:
     def to_function(self, stage, name, args, results, *margs):
         return self.opti.to_function(name, [stage.value(a) for a in args], results, *margs)
 
+    def fill_placeholders_integral(self, phase, stage, expr, *args):
+        if phase==1:
+            I = stage.state(quad=True)
+            stage.set_der(I, expr)
+            return stage.at_tf(I)
+
+    def fill_placeholders_T(self, phase, stage, expr, *args):
+        if phase==1:
+            if isinstance(stage._T, FreeTime):
+                init = stage._T.T_init
+                stage.set_T(stage.variable())
+                stage.subject_to(stage._T>=0)
+                stage.set_initial(stage._T, init,priority=True)
+                return stage._T
+            else:
+                return stage._T
+        return self.eval(stage, expr)
+
+    def fill_placeholders_t0(self, phase, stage, expr, *args):
+        if phase==1:
+            if isinstance(stage._t0, FreeTime):
+                init = stage._t0.T_init
+                stage.set_t0(stage.variable())
+                stage.set_initial(stage._t0, init,priority=True)
+                return stage._t0
+            else:
+                return stage._t0
+        return self.eval(stage, expr)
+
+    def fill_placeholders_t(self, phase, stage, expr, *args):
+        return None
+
 from casadi import substitute
 
 class OptiWrapper(Opti):
@@ -212,7 +249,7 @@ class OptiWrapper(Opti):
             self.initial_keys.append(key)
             self.initial_values.append(value)
 
-    def transcribe_placeholders(self,placeholders):
+    def transcribe_placeholders(self,phase,placeholders):
         Opti.subject_to(self)
         n_constr = len(self.constraints)
         res = placeholders([c[0] for c in self.constraints] + [self.objective]+self.initial_keys)
@@ -240,4 +277,7 @@ class OptiSolWrapper:
 
     def value(self, expr, *args,**kwargs):
         placeholders = self.opti_wrapper.ocp.placeholders_transcribed
-        return self.sol.value(placeholders(expr), *args, **kwargs)
\ No newline at end of file
+        return self.sol.value(placeholders(expr), *args, **kwargs)
+
+    def stats(self):
+        return self.sol.stats()
\ No newline at end of file
