@@ -235,8 +235,10 @@ class SamplingMethod(DirectMethod):
         self.X = []  # List that will hold N+1 decision variables for state vector
         self.U = []  # List that will hold N decision variables for control vector
         self.Z = []  # Algebraic vars
-        self.T = None
+
+        self.T = None # Will be set to numbers/opti.variables
         self.t0 = None
+
         self.P = []
         self.V = None
         self.V_control = []
@@ -330,17 +332,17 @@ class SamplingMethod(DirectMethod):
         res = I.call({'x0': X, 'p': vertcat(U, DT, P, t0)})
         return Function('F', [X, U, t0, DT, P], [res["xf"], MX(), res["qf"], res["zf"], MX()], ['x0', 'u', 't0', 'DT', 'p'], ['xf', 'poly_coeff','qf','zf','poly_coeff_z'])
 
-    def transcribe_placeholders(self, stage, placeholders):
+    def transcribe_placeholders(self, phase, stage, placeholders):
         """
         Transcription is the process of going from a continuous-time OCP to an NLP
         """
-        return stage._transcribe_placeholders(self, placeholders)
+        return stage._transcribe_placeholders(phase, self, placeholders)
 
-    def transcribe(self, stage, pass_nr=1,**kwargs):
+    def transcribe(self, stage, phase=1,**kwargs):
         """
         Transcription is the process of going from a continuous-time OCP to an NLP
         """
-        if pass_nr>1: return
+        if phase>1: return
         opti = stage.master._method.opti
         DM.set_precision(14)
         self.add_variables(stage, opti)
@@ -354,10 +356,8 @@ class SamplingMethod(DirectMethod):
         self.add_constraints(stage, opti)
         self.add_constraints_after(stage, opti)
         self.add_objective(stage, opti)
-    
-        # Create time grid (might be symbolic)
-        self.T = self.eval(stage, stage._T)
-        self.t0 = self.eval(stage, stage._t0)
+
+
 
         self.set_initial(stage, opti, stage._initial)
         T_init = opti.debug.value(self.T, opti.initial())
@@ -377,6 +377,7 @@ class SamplingMethod(DirectMethod):
         self.set_initial(stage, opti, initial)
         self.set_parameter(stage, opti)
 
+
     def add_constraints_before(self, stage, opti):
         for c, meta, _ in stage._constraints["point"]:
             e = self.eval(stage, c)
@@ -421,14 +422,16 @@ class SamplingMethod(DirectMethod):
             opti.subject_to(self.eval_at_control(stage, c_spline, k), meta=meta)
         except IndexError:
             pass
-    def fill_placeholders_integral_control(self, stage, expr, *args):
+    def fill_placeholders_integral_control(self, phase, stage, expr, *args):
+        if phase==1: return
         r = 0
         for k in range(self.N):
             dt = self.control_grid[k + 1] - self.control_grid[k]
             r = r + self.eval_at_control(stage, expr, k)*dt
         return r
 
-    def fill_placeholders_sum_control(self, stage, expr, *args):
+    def fill_placeholders_sum_control(self, phase, stage, expr, *args):
+        if phase==1: return
         r = 0
         for k in range(self.N):
             r = r + self.eval_at_control(stage, expr, k)
@@ -437,18 +440,14 @@ class SamplingMethod(DirectMethod):
     def placeholders_next(self, stage, expr, *args):
         self.eval_at_control(stage, expr, k+1)
 
-    def fill_placeholders_at_t0(self, stage, expr, *args):
+    def fill_placeholders_at_t0(self, phase, stage, expr, *args):
+        if phase==1: return
         return self.eval_at_control(stage, expr, 0)
 
-    def fill_placeholders_at_tf(self, stage, expr, *args):
+    def fill_placeholders_at_tf(self, phase, stage, expr, *args):
+        if phase==1: return
         return self.eval_at_control(stage, expr, -1)
 
-    def fill_placeholders_t0(self, stage, expr, *args):
-        return self.t0
-
-    def fill_placeholders_T(self, stage, expr, *args):
-        return self.T
-
     def add_objective(self, stage, opti):
         opti.add_objective(self.eval(stage, stage._objective))
 
@@ -507,7 +506,7 @@ class SamplingMethod(DirectMethod):
         return vertcat(vvcat(self.P), self.get_p_control_at(stage, k), self.V, self.get_v_control_at(stage, k))
 
     def eval(self, stage, expr):
-        return stage.master._method.eval_top(stage.master, stage._expr_apply(expr, p=veccat(*self.P), v=self.V, t0=self.t0, T=self.T))
+        return stage.master._method.eval_top(stage.master, stage._expr_apply(expr, p=veccat(*self.P), v=self.V, t0=stage.t0, T=stage.T))
 
     def eval_at_control(self, stage, expr, k):
         try:
@@ -624,7 +623,7 @@ class SamplingMethod(DirectMethod):
             if is_equal(parameter, p):
                 found = True
                 opti.set_value(hcat(self.P_control[i]), value)
-        assert found, "Parameter not found"
+        assert found, "You attempted to set the value of a non-parameter."
 
     def add_parameter(self, stage, opti):
         for p in stage.parameters['']:
@@ -634,6 +633,6 @@ class SamplingMethod(DirectMethod):
 
     def set_parameter(self, stage, opti):
         for i, p in enumerate(stage.parameters['']):
-            opti.set_value(self.P[i], stage._param_vals[p])
+            opti.set_value(self.P[i], stage._param_value(p))
         for i, p in enumerate(stage.parameters['control']):
-            opti.set_value(hcat(self.P_control[i]), stage._param_vals[p])
+            opti.set_value(hcat(self.P_control[i]), stage._param_value(p))
\ No newline at end of file
