@@ -699,21 +699,45 @@ class Stage:
             self.master._transcribed_placeholders.mark_dirty()
         return r
 
+
+
+                
     def _transcribe_placeholders(self, phase, method, placeholders):
+
+        def normalize(out):
+            if out is None:
+                return None
+            if isinstance(out, dict):
+                return out
+            return {"normal": out} 
+
+        def prefix(d, prefix=None):
+            if prefix is None or d is None:
+                return d
+            ret = {}
+            for k,v in d.items():
+                ret[prefix+"."+k] = v
+            return ret
+
+        def do(tag=None):
+            ret = prefix(normalize(callback(phase, self, expr)),tag)
+            if ret is not None:
+                placeholders[phase][symbol] = ret
+                if phase==2 and isinstance(expr,MX) and expr.is_symbolic() and symbol in placeholders[phase-1]:
+                    placeholders[phase][expr] = ret
         # Fixed-point iteration:
         # Phase 1 may introduce extra placeholders
         while True:
             len_before = len(self._placeholders)
-            for k, (n,v) in list(self._placeholders.items()):
-                if k not in placeholders[phase]:
-                    callback = getattr(method, 'fill_placeholders_' + n)
-                    if phase==2 and k in placeholders[phase-1]:
-                        v = placeholders[phase-1][k]
-                    ret = callback(phase, self, v)
-                    if ret is not None:
-                        placeholders[phase][k] = ret
-                        if phase==2 and isinstance(v,MX) and v.is_symbolic() and k in placeholders[phase-1]:
-                            placeholders[phase][v] = ret
+            for symbol, (species,expr) in list(self._placeholders.items()):
+                if symbol not in placeholders[phase]:
+                    callback = getattr(method, 'fill_placeholders_' + species)
+                    if phase==2 and symbol in placeholders[phase-1]:
+                        for tag, expr in placeholders[phase-1][symbol].items():
+                            do(tag)
+                            continue
+                    do()
+
             len_after = len(self._placeholders)
             if len_before==len_after: break
 
@@ -829,6 +853,15 @@ class Stage:
     def _augmented(self):
         return self._var_augmented if self._var_augmented else self
 
+    @property
+    def _transcribed(self):
+        if not self.is_transcribed:
+            self.master._transcribe()
+        if self._is_original:
+            return self._augmented 
+        else:
+            return self
+
     def _set_transcribed(self, val):
         if self.master:
             if self._is_original:
@@ -967,6 +1000,7 @@ class Stage:
             include_last = False
         return grid, include_first, include_last
 
+    @transcribed
     def sample(self, expr, grid='control', **kwargs):
         """Sample expression symbolically on a given grid.
 
@@ -996,7 +1030,6 @@ class Stage:
         >>> sol = ocp.solve()
         >>> tx, xs = sol.sample(x, grid='control')
         """
-        self.master._transcribe()
         placeholders = self.master.placeholders_transcribed
         grid, include_first, include_last = self._parse_grid(grid)
         kwargs["include_first"] = include_first
@@ -1074,8 +1107,8 @@ class Stage:
             raise Exception(msg)
         N, M = stage._method.N, stage._method.M
 
-        expr_f = Function('expr', [stage.t, stage.x, stage.z, stage.u], [expr])
-        assert not expr_f.has_free()
+        expr_f = Function('expr', [stage.t, stage.x, stage.z, stage.u, vertcat(stage.p, stage.v), stage.t0, stage.T], [expr])
+        assert not expr_f.has_free(), str(expr_f.free_mx())
 
         time = stage._method.control_grid
         total_time = []
@@ -1097,7 +1130,9 @@ class Stage:
                     z = mtimes(coeff_z,tpower_z)
                 else:
                     z = nan
-                sub_expr.append(stage._method.eval_at_integrator(stage, expr_f(local_t.T, mtimes(coeff,tpower), z, stage._method.U[k]), k, l))
+
+                pv = stage._method.get_p_sys(stage,k)
+                sub_expr.append(stage._method.eval_at_integrator(stage, expr_f(local_t.T, mtimes(coeff,tpower), z, stage._method.U[k], pv, stage._method.t0, stage._method.T), k, l))
                 t0+=dt
 
         ts = tlocal[-1,:]
@@ -1109,10 +1144,12 @@ class Stage:
         else:
             z = nan
 
-        sub_expr.append(stage._method.eval_at_integrator(stage, expr_f(time[k+1], mtimes(stage._method.poly_coeff[-1],tpower), z, stage._method.U[-1]), k, l))
+        pv = stage._method.get_p_sys(stage,-1)
+        sub_expr.append(stage._method.eval_at_integrator(stage, expr_f(time[k+1], mtimes(stage._method.poly_coeff[-1],tpower), z, stage._method.U[-1], pv, stage._method.t0, stage._method.T), k, l))
 
         return vcat(total_time), hcat(sub_expr)
 
+    @transcribed
     def value(self, expr):
         """Get the value of an (non-signal) expression.
 
@@ -1121,14 +1158,15 @@ class Stage:
         expr : :obj:`casadi.MX`
             Arbitrary expression containing no signals (states, controls) ...
         """
-        self.master._transcribe()
         placeholders = self.master.placeholders_transcribed
         return placeholders(self._method.eval(self, expr))
 
+    @transcribed
     def discrete_system(self):
         """Hack"""
         return self._method.discrete_system(self)
 
+    @transcribed
     def sampler(self, *args):
         """Returns a function that samples given expressions
 
@@ -1170,8 +1208,6 @@ class Stage:
         if not isinstance(exprs, list):
             ret_list = False
             exprs = [exprs]
-
-        self.master._transcribe()
         t = MX.sym('t')
 
         """Evaluate expression at extra fine integrator discretization points."""
